/*
                                  IPSTB Project
                         ---------------------------------


   Copyright (C) 2005 Koninklijke Philips Electronics N.V.,
   All Rights Reserved.

Filename: analog_backend.c


Rev Date       Author      Comments
--------------------------------------------------------------------------------
  2 20050531   batelaan    Updated copyright.
003 20051123   raiyat      Linux 2.6.14.2 changes
004 20060908   laird       Linux 2.6.17.7 changes
*/
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/kdev_t.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#define THIS_MODULE_DESCRIPTION "Analog Backend driver"
MODULE_DESCRIPTION(THIS_MODULE_DESCRIPTION);
MODULE_AUTHOR("Mike Neill");
MODULE_LICENSE("GPL");

#include <linux/i2c.h>
#include <linux/i2c-dev.h>

#include "analog_backend.h"

static int debug = 0;
module_param(debug, int, 0);
MODULE_PARM_DESC(debug, "Debug level (0-1)");

#define dprintk(num, format, args...) \
	do { \
		if (debug >= num) \
			printk(format, ##args); \
	} while (0)

static struct class *ana_backend_class;

struct class *ana_backend_get_class(void)
{
	return ana_backend_class;
}


struct class_device *ana_backend_device_create(struct class *cls,
                                               struct class_device *parent,
                                               dev_t devt,
                                               struct device *device,
                                               char *devname)
{
    return class_device_create(cls, parent, devt, device, devname);
}


void ana_backend_device_remove(struct class *cls,
                               dev_t devt)
{
    return class_device_destroy(cls, devt);
}

static int __init ana_backend_init (void)
{
	ana_backend_class = class_create( THIS_MODULE, "analog_backend" );
	printk ("%s (%s-%s)\n", THIS_MODULE_DESCRIPTION, __DATE__, __TIME__);
	return 0;
}

static void __exit ana_backend_exit (void)
{
	class_destroy(ana_backend_class);
	printk("Shutting down %s\n", THIS_MODULE_DESCRIPTION);
}

module_init(ana_backend_init);
module_exit(ana_backend_exit);

EXPORT_SYMBOL(ana_backend_get_class);
EXPORT_SYMBOL(ana_backend_device_create);
EXPORT_SYMBOL(ana_backend_device_remove);
