#ifndef PHSTBRPC_INTERNAL_H    /* Multi-include protection.   <FILENAME>_H */
#define PHSTBRPC_INTERNAL_H

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright (C) 2006 NXP B.V.
 * All Rights Reserved.
*//**
    @file   phStbRpc_Internal.h
    @brief  RPC internal definitions.

    NOTE: To be used only by the phStbRpc component itself.

@b  Component: phStbRpc.

*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20050510    batelaan    Original
  2 20050513    batelaan    Crosscheck fixes
  3 20050608    batelaan    Removed dumpdp stuff.
  4 20050704    batelaan    Add some more functions.
  5 20050719    batelaan    Add phStbRpc_InternalFun_UnitTest_Sleep
  6 20050804    batelaan    Add phStbRpc_InternalFun_Cmd
  7 20050919    burningh    Remove phStbRpc_InternalFun_DbgCmd
  8 20060124    batelaan    Add extern "C" for use with C++
  9 20070730    laird       Updated Header
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/


/*******************
* INCLUDE FILES    *
********************/

#include <phStbCommon.h>
#include <phStbRpc_Internal.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**************************************************************
* EXPORTED MACROS         PH_STBRPC_<_WORD>+                  *
***************************************************************/

/** Maximum payload length for the phStbRpc_HandlerId_Internal RPC handler. */
#define PH_STBRPC_INTERNAL_PAYLOADLENGTH 256

/**************************************************************
* EXPORTED TYPEDEFS       [p]phStbRpc_<Word>+_t               *
***************************************************************/

typedef enum phStbRpc_InternalFun {
    /** Function id for handler ::phStbRpc_HandlerId_Internal which executes a command line
        (using tmMUtiCmd). Uses ::phStbRpc_Option_Async.
    */
    phStbRpc_InternalFun_Cmd = 0,

#ifdef UNITTEST
    /** For testing. Appends RPCDONE to its argument. */
    phStbRpc_InternalFun_UnitTest_Appends,
    /** For testing. Increments (arg!=0) or resets (arg==0) a counter.
        Returns old value of counter in buffer. */
    phStbRpc_InternalFun_UnitTest_UpdateCounter,
    /** For testing. Returns value of counter as return result. */
    phStbRpc_InternalFun_UnitTest_GetCounter,
    /** Sleeps for N seconds. */
    phStbRpc_InternalFun_UnitTest_Sleep,
#endif

    /** Number of rpc entries for handler ::phStbRpc_HandlerId_Internal. */
    phStbRpc_InternalFun_Count

} phStbRpc_InternalFun_t;

/*******************************************************************
* EXPORTED DATA      g[k|p|kp|pk|kpk]phStbRpc_<Word>+              *
********************************************************************/

/*******************************************************************
* EXPORTED FUNCTION PROTOTYPES     phStbRpc[_<Word>+]              *
********************************************************************/

#if TMFL_CPU_IS_TM
/** Start handler for ::phStbRpc_HandlerId_Internal. */
extern Void phStbRpc_InternalHandlerInit(Void);
#endif

#ifdef UNITTEST
/** Invoke unit test. */
extern Void phStbRpc_UnitTest(Void);
#endif

#ifdef __cplusplus
}
#endif

#endif /* PHSTBRPC_INTERNAL_H      Do not add any thing below this line */
